#!/usr/bin/perl
#
# //////
# --@-@  Geeksen's Laboratory
#     >  Simplicity matters.
# ____/  http://www.geeksen.com
#
# Copyright (c) 1997-2008 Seung-min Yoon
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

use strict;

# Conf
my($locale) = 'en';                  # ko, en ..
my($charset) = 'euc-kr';             # euc-kr, ks_c_5601_1987, us-ascii ..
my($timezone) = '+0900';             # +0900, KST, GMT ..
my($sendmail) = '/usr/bin/sendmail';
my($useSMTP) = 1;

# Size
my($width) = '770';
my($pagesize) = 10;
my($pagelink) = 10;

# Hashes
my(%m); # mail
my(%l); # locale

local(*N);
local(*M);

# HTML Head
sub Head {
  my($title) = $_[0];

  print <<EOT;
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/1999/REC-html401-19991224/strict.dtd">

<html>
<head>
  <title>Geeksen Mail</title>
  <meta http-equiv='Content-Type' content='text/html; charset=$charset'>
  <style type='text/css'>
    body { background-color:#ffffff; color:#666666; font-size:9pt; font-family:arial; line-height:14pt; }
    th { background-color:#666666; color:#ffffff; font-size:9pt; font-family:arial; line-height:14pt; }
    td { background-color:#ffffff; color:#666666; font-size:9pt; font-family:arial; line-height:14pt; }
    .large { color:#666666; font-size:14pt; font-family:arial; }
    .list_bg1 { background-color:#ffffff; }
    .list_bg2 { background-color:#eeeeee; }
    .mail_body {
EOT

  if($m{'bgcolor'} ne '') { print "      background-color:$m{'bgcolor'};\n"; }
  if($m{'background'} ne '') { print "      background-image:url($m{'background'});\n"; }

  print <<EOT;
    }
    a:link { color:#0066cc; text-decoration:none; }
    a:active { color:#0066cc; text-decoration:none; }
    a:visited { color:#0066cc; text-decoration:none; }
    a:hover { color:#0066cc; text-decoration:underline; }
  </style>
</head>

<body>

<table border=0 width='$width'>
  <tr>
    <td><span class='large'>$title</span></td>
EOT
}

# HTML Tail
sub Tail {
  print <<EOT;

</body>
</html>
EOT
}

use CGI;
use Socket;

my($q) = new CGI;
my($buf, $total);
my($script, $domain);

$buf = '';
$total = 0;

$script = $ENV{'SCRIPT_NAME'};
$domain = '.' . $ENV{'HTTP_HOST'};

my($num) = $q->param('num');
my($mode) = $q->param('mode');
my($type) = $q->param('type');
my($back) = int($q->param('back'));
my($page) = int($q->param('page'));
my($time) = time();
my($email) = '';
my($userid) = $q->cookie(-name=>'mUSERID');
my($passwd) = unpack('u', $q->cookie(-name=>'mPASSWD'));
my($server) = $q->cookie(-name=>'mSERVER');
my($mimepart) = $q->param('mimepart');

if($back > -1) { $back++; }

if($q->param('userid') ne '') {
   $userid = $q->param('userid');
   $passwd = $q->param('passwd');
   $server = $q->param('server');
}

if($userid =~ /\@/) { $email = $userid; }
else { $email = $userid . '@' . $server; }

&mLocale;

   if($mode eq 'list')     { &List;     }
elsif($mode eq 'read')     { &Read;     }
elsif($mode eq 'delete')   { &Delete;   }
elsif($mode eq 'form')     { &Form;     }
elsif($mode eq 'send')     { &Send;     }
elsif($mode eq 'download') { &Download; }
elsif($mode eq 'logout')   { &Logout;   }
else { &LoginForm; }

sub List {
  my($cookie1, $cookie2, $cookie3);
  my($i, $j, $totalpage, $first, $last);

  &mLOGIN;
  &mSTAT;

  $totalpage = int(($total - 1) / $pagesize) + 1;
  if($page == 0) { $page = 1; }
  if($totalpage == 0) { $totalpage = 1; }
  if($page > $totalpage) { $page = $totalpage; }

  if($q->param('userid') ne '') {
    $cookie1 = $q->cookie(-name=>'mUSERID', -value=>$userid, -path=>'/', -domain=>$domain);
    $cookie2 = $q->cookie(-name=>'mPASSWD', -value=>pack('u', $passwd), -path=>'/', -domain=>$domain);
    $cookie3 = $q->cookie(-name=>'mSERVER', -value=>$server, -path=>'/', -domain=>$domain);

    print $q->header(-charset=>$charset, -cookie=>[$cookie1, $cookie2, $cookie3]);
  }
  else { print $q->header(-charset=>$charset); }

  &Head($email);
  print <<EOT;
    <td align=right>[ <a href='$script?mode=logout'>$l{'logout'}</a> ]</td>
  </tr>
</table>

<script type='text/javascript'>
function CheckAll() {
  for(i = 0; i < document.forms[0].elements.length; i++) {
    if(document.forms[0].elements[i].name.indexOf('mail_') == 0 &&
      document.forms[0].elements[i].type.toUpperCase() == 'CHECKBOX') {
      document.forms[0].elements[i].checked = !document.forms[0].elements[i].checked;
    }
  }
}
</script>

<form method=post action='$script'>
<table border=0 width='$width'>
  <tr>
    <td>[ <a href='$script?mode=form&amp;type=compose&amp;page=$page'>$l{'compose'}</a> ]</td>
    <td align=right>$l{'total'} : $total, $l{'page'} : $page / $totalpage</td>
  </tr>
</table>

<table border=0 width='$width' cellpadding=4>
  <tr>
    <th>
      <input type=checkbox name=mark1 onClick='CheckAll();'>
      $l{'no'}
      <input type=hidden name=mode value='delete'>
      <input type=hidden name=page value='$page'>
    </th>
    <th>$l{'from'}</th>
    <th>$l{'subject'}</th>
    <th>$l{'date'}</th>
  </tr>
EOT

  $first = $total - ($page - 1) * $pagesize;
  if($first > $pagesize) { $last = $first - $pagesize + 1; }
  else { $last = 1; }

  for($i = $first; $i >= $last; $i--) {
    &mTOP($i);

    if(($i % 2) == 0) {
      print <<EOT;
  <tr>
    <td class='list_bg1'>
      <input type=checkbox name=mail_$i value='1'>
      $i $m{'priority'} $m{'attachment'}
    </td>
    <td class='list_bg1'>$m{'from'}</td>
    <td class='list_bg1'><a href='$script?mode=read&amp;num=$i&amp;page=$page'>$m{'subject'}</a></td>
    <td class='list_bg1'>$m{'date'}</td>
  </tr>
EOT
    }
    else {
      print <<EOT;
  <tr>
    <td class='list_bg2'>
      <input type=checkbox name=mail_$i value='1'>
      $i $m{'priority'} $m{'attachment'}
    </td>
    <td class='list_bg2'>$m{'from'}</td>
    <td class='list_bg2'><a href='$script?mode=read&amp;num=$i&amp;page=$page'>$m{'subject'}</a></td>
    <td class='list_bg2'>$m{'date'}</td>
  </tr>
EOT
    }
  }
  &mQUIT;

  print "  <tr>\n";
  print "    <td><input type=checkbox name=mark2 onClick='CheckAll();'></td>\n";
  print "    <td align=center colspan=3>\n";
  $i = int(($page - 1) / $pagelink) * $pagelink + 1;
  if($i > $pagelink) {
    print  "      [<a href='$script?mode=list'>1</a>]\n";
    printf "      [<a href='$script?mode=list&amp;page=%d'>$l{'prev'}</a>]\n", $i - 1;
  }
  for($j = 0; $i <= $totalpage && $j < $pagelink; $i++, $j++) {
    if($i == $page) { print "      [$i]\n"; }
    else { print "      [<a href='$script?mode=list&amp;page=$i'>$i</a>]\n"; }
  }
  if($i <= $totalpage) {
    print "      [<a href='$script?mode=list&amp;page=$i'>$l{'next'}</a>]\n";
    print "      [<a href='$script?mode=list&amp;page=$totalpage'>$totalpage</a>]\n";
  }

  print <<EOT;
    </td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td><hr></td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td><input type=submit value='$l{'delete_submit'}'></td>
    <td align=right>[
      <a href='$script?mode=list&amp;page=$page&amp;time=$time'>$l{'reload'}</a> |
EOT

  if($page > 1) {
    printf "      <a href='$script?mode=list&amp;page=%d'>$l{'prev'}</a> |\n", $page - 1;
  }
  else { print "      $l{'prev'} |\n"; }

  if($page < $totalpage) {
    printf "      <a href='$script?mode=list&amp;page=%d'>$l{'next'}</a> ]\n", $page + 1;
  }
  else { print "      $l{'next'} ]\n"; }

  print "    </td>\n  </tr>\n</table>\n</form>\n";
  &Tail;
}

sub Read {
  my($i);
  my($attachment_name, $attachment_type, $attachment_size);

  &mLOGIN;
  &mSTAT;
  &mRETR($num, $mimepart);
  &mQUIT;

  print $q->header(-charset=>$charset);
  &Head($m{'subject'});
  print <<EOT;
    <td align=right>[ <a href='$script?mode=logout'>$l{'logout'}</a> ]</td>
  </tr>
</table>

<form>
<table border=0 width='$width'>
  <tr>
    <td>[
      <a href='$script?mode=form&amp;type=compose&amp;page=$page'>$l{'compose'}</a> |
      <a href='$script?mode=form&amp;type=reply&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'reply'}</a> |
      <a href='$script?mode=form&amp;type=reply_all&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'reply_all'}</a> |
      <a href='$script?mode=form&amp;type=forward&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'forward'}</a> |
      <a href='$script?mode=read&amp;type=headers&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart&amp;back=$back'>$l{'headers'}</a> |
EOT

  if($mimepart eq '') {
    print "      <a href='$script?mode=delete&amp;num=$num&amp;page=$page&amp;time=$time&amp;back=-1'>$l{'delete'}</a> ]\n";
  }
  else { print "      $l{'delete'} ]\n"; }

  print <<EOT;
    </td>
    <td align=right>[
EOT

  if($num < $total) {
    printf "      <a href='$script?mode=read&amp;num=%d&amp;page=$page&amp;back=$back'>$l{'prev'}</a> |\n", $num + 1;
  }
  else { print "      $l{'prev'} |\n"; }

  if($num > 1) {
    printf "      <a href='$script?mode=read&amp;num=%d&amp;page=$page&amp;back=$back'>$l{'next'}</a> |\n", $num - 1;
  }
  else { print "      $l{'next'} |\n"; }

  if($back == -1) {
    print "      <a href='$script?mode=list&amp;page=$page&amp;time=$time'>$l{'list'}</a> ]\n";
  }
  else { print "      <a href='JavaScript:history.go(-$back);'>$l{'list'}</a> ]\n"; }

  print <<EOT;
    </td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td><hr></td>
  </tr>
</table>

<table border=0 width='$width' cellpadding=4>
  <tr>
    <th>$l{'no'}</th>
    <td>$num / $total</td>
  </tr>
  <tr>
    <th>$l{'from'}</th>
    <td>$m{'from'}</td>
  </tr>
  <tr>
    <th>$l{'to'}</th>
    <td>$m{'to'}</td>
  </tr>
EOT

  if($m{'cc'} ne '') {
    print "  <tr>\n    <th>";
    print "$l{'cc'}</th>\n";
    print "    <td>$m{'cc'}</td>\n";
    print "  </tr>\n";
  }

  print <<EOT;
  <tr>
    <th>$l{'date'}</th>
    <td>$m{'date'}</td>
  </tr>
  <tr>
    <th>$l{'subject'}</th>
    <td>$m{'subject'}</td>
  </tr>
EOT

  if($m{'attachment.count'} > 0) {
    print "  <tr>\n    <th>";
    print "$l{'attachment'}</th>\n";
    print "    <td>";
    for($i = 0; $i < $m{'attachment.count'}; $i++) {
      $attachment_name = $m{'attachment.' . $i . '.name'};
      $attachment_size = &mSize($m{'attachment.' . $i . '.size'});
      $attachment_type = $m{'attachment.' . $i . '.type'};

      if($attachment_type =~ /message\/rfc822/i) {
        print "<a href='$script?mode=read&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart\.$i&amp;back=$back'>$attachment_name ($attachment_size)</a>";
      }
      else {
        $attachment_name =~ s/ /%20/g;
        print "<a href='$script?mode=download&amp;num=$num&amp;mimepart=$mimepart&amp;time=$time&amp;attachment=$i/$attachment_name'>";
        $attachment_name =~ s/%20/ /g;
        print "$attachment_name ($attachment_size)</a>";
      }
      if($i < $m{'attachment.count'} - 1) { print ", "; }
    }
    print "    </td>\n  </tr>\n";
  }

  print <<EOT;
</table>

<table border=0 width='$width'>
  <tr>
    <td><hr></td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
EOT

  if($type ne 'headers') {
    print "    <td class='mail_body'>\n";
  }
  else { print "    <td>\n"; }

  if($type eq 'headers') { print "<pre>$m{'headers'}</pre>"; }
  else { print "$m{'body'}"; }

  print <<EOT;
    </td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td><hr></td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td>[
      <a href='$script?mode=form&amp;type=compose&amp;page=$page'>$l{'compose'}</a> |
      <a href='$script?mode=form&amp;type=reply&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'reply'}</a> |
      <a href='$script?mode=form&amp;type=reply_all&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'reply_all'}</a> |
      <a href='$script?mode=form&amp;type=forward&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart'>$l{'forward'}</a> |
      <a href='$script?mode=read&amp;type=headers&amp;num=$num&amp;page=$page&amp;mimepart=$mimepart&amp;back=$back'>$l{'headers'}</a> |
EOT

  if($mimepart eq '') {
    print "      <a href='$script?mode=delete&amp;num=$num&amp;page=$page&amp;time=$time&amp;back=-1'>$l{'delete'}</a> ]\n";
  }
  else { print "      $l{'delete'} ]\n"; }

  print <<EOT;
    </td>
    <td align=right>[
EOT

  if($num < $total) {
    printf "      <a href='$script?mode=read&amp;num=%d&amp;page=$page&amp;back=$back'>$l{'prev'}</a> |\n", $num + 1;
  }
  else { print "      $l{'prev'} |\n"; }

  if($num > 1) {
    printf "      <a href='$script?mode=read&amp;num=%d&amp;page=$page&amp;back=$back'>$l{'next'}</a> |\n", $num - 1;
  }
  else { print "      $l{'next'} |\n"; }

  if($back == -1) {
    print "      <a href='$script?mode=list&amp;page=$page&amp;time=$time'>$l{'list'}</a> ]\n";
  }
  else { print "      <a href='JavaScript:history.go(-$back);'>$l{'list'}</a> ]\n"; }

  print "    </td>\n  </tr>\n</table>\n</form>";
  &Tail;
}

sub Delete {
  my($i) = 0;
  my($name);

  &mLOGIN;
  &mSTAT;
  if($num > 0) { &mDELE($num); }
  else {
    for($i = 1; $i <= $total; $i++) {
      $name = 'mail_' . $i;
      if($q->param($name) == 1) { &mDELE($i); }
    }
  }

  &mQUIT;
  print $q->header(-charset=>$charset);
  if($num > 1) {
    $num--;
    &Reload("$script?mode=read&amp;num=$num&amp;page=$page&amp;time=$time&amp;back=-1");
  }
  else { &Reload("$script?mode=list&amp;page=$page"); }
}

sub Form {
  my($to, $cc, $subject, $tmp, $i);
  my(@tmp);
  $to = $q->param('to');
  $subject = $q->param('subject');

  if($type ne 'compose') {
    &mLOGIN;
    &mRETR($num, $mimepart);
    &mQUIT;

    if($type =~ /^reply/) {
      $num = 0;

      $to = $m{'from'};
      $to = &mDecodeHTML($to);
      if($to =~ /<(.*)>/) { $to = $1; }

      $cc =  $m{'to'} . ', ' . $m{'cc'};
      $cc =  &mDecodeHTML($cc);
      $cc =~ s/$email//g;
      if($type eq 'reply') { $cc = ''; }
      else {
        @tmp = split(/\,/, $cc);
        $cc = '';

        for($i = 0; $i <= $#tmp; $i++) {
          if($tmp[$i] =~ /$email/) { next; }

          $cc .= $tmp[$i];
          if($i < $#tmp) { $cc = ', '; }
        }
      }
      $subject = 'RE: ' . $m{'subject'};
    }
    else { $subject = 'FW: ' . $m{'subject'}; }
  }

  print $q->header(-charset=>$charset);
  &Head($l{$type});
  print <<EOT;
    <td align=right>[ <a href='$script?mode=logout'>$l{'logout'}</a> ]</td>
  </tr>
</table>

<form method=post action='$script' enctype='multipart/form-data'>
<table border=0>
  <tr>
    <th>$l{'sender'}</th>
    <td>
    <input type=text name=from size=40 value='<$email>'>
    <input type=hidden name=mode value='send'>
    <input type=hidden name=num value='$num'>
    <input type=hidden name=page value='$page'>
    </td>
  </tr>
  <tr>
    <th>$l{'to'}</th>
    <td><input type=text name=to size=40 value='$to'></td>
  </tr>
  <tr>
    <th>$l{'cc'}</th>
    <td><input type=text name=cc size=40 value='$cc'></td>
  </tr>
  <tr>
    <th>$l{'bcc'}</th>
    <td><input type=text name=bcc size=40></td>
  </tr>
  <tr>
    <th>$l{'subject'}</th>
    <td><input type=text name=subject size=40 value='$subject'>
EOT

  if($useSMTP == 1) {
    print <<EOT;
    </td>
  </tr>
  <tr>
    <th>$l{'smtp'}</th>
    <td><input type=text name=smtp size=40 value='$server'></td>
  </tr>
EOT
  }

  print <<EOT;
  <tr>
    <td colspan=2><textarea name=body rows=20 cols=75>
EOT

  if($type =~ /^reply/) {
    print "\n\n";
    print "> ---------- $l{'original_message'} ----------\n";
    print "> $l{'from'} : $m{'from'}\n";
    print "> $l{'to'} : $m{'to'}\n";
    if($m{'cc'} ne '') { print "> $l{'cc'} : $m{'cc'}\n"; }
    print "> $l{'date'} : $m{'date'}\n";
    print "> $l{'subject'} : $m{'subject'}\n>\n> ";

    $m{'body'} =~ s/<br>/\r\n/gi;
    $m{'body'} =~ s/<\/?([!A-Za-z])[^>]*\>//gi;
    $m{'body'} =~ s/\r\n\r\n/\r\n/g;
    $m{'body'} =~ s/\r\n/\r\n> /g;
    print $m{'body'};
  }
  elsif($type eq 'forward') {
    print $l{'original_message_included'};
  }

  print <<EOT;
</textarea>
    </td>
  </tr>
  <tr>
    <th>$l{'attachment'}</th>
    <td><input type=file name=file1 size=40></td>
  </tr>
  <tr>
    <th>$l{'attachment'}</th>
    <td><input type=file name=file2 size=40></td>
  </tr>
  <tr>
    <td colspan=2><hr></td>
  </tr>
  <tr>
    <td><input type=submit value='$l{'send_submit'}'></td>
    <td align=right><input type=button value='$l{'cancel_button'}' onClick='history.back();'></td>
  </tr>
</table>
</form>
EOT
  &Tail;
}

sub Send {
  my($encoding, $boundary, $boundary_alternative);
  my($filename1, $filename2, $filename, $filetype, $file, $tmp);
  my($from, $to, $cc, $bcc, $subject, $date, $smtp, $body);

  $from = $q->param('from');
  $to = $q->param('to');
  $cc = $q->param('cc');
  $bcc = $q->param('bcc');
  $subject = $q->param('subject');
  $date = &mGetDate;
  $smtp = $q->param('smtp');
  $body = $q->param('body');
  $filename1 = $q->param('file1');
  $filename2 = $q->param('file2');
  $boundary = 'MultiPart_Boundary_' . $time;
  $boundary_alternative = 'MultiPart_Alternative_Boundary_' . $time;

  if($from eq '')    { &Error($l{'from_error'}); }
  if($to eq '')      { &Error($l{'to_error'}); }
  if($subject eq '') { $subject = $l{'subject_unspecified'}; }
  if($body eq '')    { &Error($l{'body_error'}); }
  if($useSMTP == 1 && $smtp eq '') { &Error($l{'smtp_error'}); }

  $encoding = '7bit';
  $body =~ s/\r\n\./\r\n \./g;

  if($subject =~ /[\xA1-\xFE]/) {
    $subject = &mEncodeBase64($subject);
    $subject =~ s/\r\n//g;
    $subject = '=?' . $charset . '?B?' . $subject . '?=';
  }

  if($body =~ /[\xA1-\xFE]/) {
    $encoding = 'base64';
    $body = &mEncodeBase64($body);
  }

  if($num > 0) {
    &mLOGIN;
    &mRETR($num, $mimepart);
    &mQUIT;
  }

  if($useSMTP == 1) {
    my($host, $sin, $buf);

    socket(N, PF_INET, SOCK_STREAM, getprotobyname('tcp'));

    $host = 0;
    $host = gethostbyname($smtp);
    if(!$host) { &Error($l{'host_error'}); }

    $sin = sockaddr_in(25, $host);
    if(!connect(N, $sin)) { &Error($l{'connect_error'}); }

    select(N);
    $| = 1;

    select(STDOUT);
    binmode(N);

    $buf = <N>;
    if($buf =~ /^[45]/) { &Error($buf); }

    print N "HELO $smtp\r\n";
    $buf = <N>;
    if($buf =~ /^[45]/) { &Error($buf); }

    print N "MAIL FROM: $from\r\n";
    $buf = <N>;
    if($buf =~ /^[45]/) { &Error($buf); }

    if($to ne '') {
      print N "RCPT TO: $to\r\n";
      $buf = <N>;
      if($buf =~ /^[45]/) { &Error($buf); }
    }

    if($cc ne '') {
      print N "RCPT TO: $cc\r\n";
      $buf = <N>;
      if($buf =~ /^[45]/) { &Error($buf); }
    }

    if($bcc ne '') {
      print N "RCPT TO: $bcc\r\n";
      $buf = <N>;
      if($buf =~ /^[45]/) { &Error($buf); }
    }

    print N "DATA\r\n";
    $buf = <N>;
    if($buf =~ /^[45]/) { &Error($buf); }
  }
  else {
    if(-x $sendmail) {
      open(N, "|$sendmail -it");
    }
    else { &Error($l{'sendmail_error'}); }
  }

  print N "Reply-to: $from\r\n";
  print N "From: $from\r\n";

  if($to ne '')  { print N "To: $to\r\n";  }
  if($cc ne '')  { print N "CC: $cc\r\n";  }
  if($bcc ne '') { print N "BCC: $bcc\r\n"; }

  print N "Subject: $subject\r\n";
  print N "Date: $date\r\n";
  print N "MIME-Version: 1.0\r\n";
  print N "X-Mailer: http://$ENV{'SERVER_NAME'}$script\r\n";
  print N "X-Sender-IP: $ENV{'REMOTE_ADDR'}\r\n";

  if($num > 0 || $filename1 ne '' || $filename2 ne '') {
    print N "Content-Type: multipart/mixed; boundary=\"$boundary\"\r\n\r\n";
  }
  else {
    print N "Content-Type: text/plain; charset=$charset\r\n";
    print N "Content-Transfer-Encoding: $encoding\r\n\r\n";
  }

  if($num > 0 || $filename1 ne '' || $filename2 ne '') {
    print N "This is a multi-part message in MIME format.\r\n\r\n";
    print N "--$boundary\r\n";
    print N "Content-Type: text/plain; charset=$charset\r\n";
    print N "Content-Transfer-Encoding: $encoding\r\n\r\n";
    print N "$body\r\n";

    foreach $filename ($filename1, $filename2) {
      if($filename eq '') { next; }
      $tmp = ''; $file = '';
      while(read($filename, $tmp, 1024)) { $file .= $tmp; }
      $filetype = $q->uploadInfo($filename)->{'Content-Type'};
      close($filename);

      if($filetype eq '') { $filetype = 'application/octet-stream'; }

      $file = &mEncodeBase64($file);
      $filename =~ s/.*\\//g;
      $filename =~ s/.*\///g;

      print N "--$boundary\r\n";
      print N "Content-Type: $filetype; name=\"$filename\"\r\n";
      print N "Content-Transfer-Encoding: base64\r\n";
      if($filetype =~ /application\/octet-stream/i) {
        print N "Content-Disposition: attachment; filename=\"$filename\"\r\n\r\n";
      }
      else { print N "Content-Disposition: inline; filename=\"$filename\"\r\n\r\n"; }
      print N "$file\r\n";
    }

    if($num > 0) {
      print N "--$boundary\r\n";
      print N "Content-Type: message/rfc822; charset=$charset\r\n";
      print N "Content-Transfer-Encoding: 7bit\r\n";
      print N "Content-Disposition: inline\r\n\r\n";
      print N "$m{'headers'}\r\n";
    }
    print N "--$boundary--\r\n";
  }
  else {
    print N "$body\r\n";
  }

  if($useSMTP == 1) {
    print N "\r\n.\r\n";
    $buf = <N>;
    if($buf =~ /^[45]/) { &Error($buf); }
    print N "QUIT\r\n";
  }
  close(N);

  print $q->header(-charset=>$charset);
  if($userid ne '') { &Reload("$script?mode=list&amp;page=$page"); }
  else { &Reload('/'); }
}

sub Download {
  my($attachment) = $q->param('attachment');
  my($tmp, $attachment_num, $attachment_name, $attachment_type, $attachment_size);

  &mLOGIN;
  &mRETR($num, $mimepart);
  &mQUIT;

  ($attachment_num, $attachment_name) = split(/\//, $attachment);
  $attachment_type =  $m{'attachment.' . $attachment_num . '.type'};
  $attachment_type =~ s/\s//g;
  $attachment_size =  $m{'attachment.' . $attachment_num . '.size'};
  binmode(STDOUT);

  if($attachment_type =~ /application\/octet-stream/i && $ENV{'HTTP_USER_AGENT'} =~ /MSIE/) {
    $attachment_type = 'application/download';
  }

  if($attachment_name =~ /\.png$/i) {
    $attachment_type = 'image/png';
  }
  elsif($attachment_name =~ /\.gif$/i) {
    $attachment_type = 'image/gif';
  }
  elsif($attachment_name =~ /\.jpg$/i) {
    $attachment_type = 'image/jpeg';
  }
  elsif($attachment_name =~ /\.txt$/i) {
    $attachment_type = 'text/plain';
  }
  elsif($attachment_name =~ /\.htm*$/i) {
    $attachment_type = 'text/html';
  }

  if($attachment_type =~ /application\/octet-stream/i) {
    print "Content-Disposition: attachment; filename=\"$attachment_name\"\n";
  }
  else { print "Content-Disposition: inline; filename=\"$attachment_name\"\n"; }

  print "Content-Transfer-Encoding: binary\n";
  print "Accept-Ranges: bytes\n";
  print "Content-Length: $attachment_size\n";
  print "Connection: close\n";
  print "Content-Type: $attachment_type\n\n";
  print $m{'attachment.' . $attachment_num};
}

sub Logout {
  my($cookie1, $cookie2, $cookie3);

  $cookie1 = $q->cookie(-name=>'mUSERID', -value=>'', -path=>'/', -domain=>$domain);
  $cookie2 = $q->cookie(-name=>'mPASSWD', -value=>'', -path=>'/', -domain=>$domain);
  $cookie3 = $q->cookie(-name=>'mSERVER', -value=>'', -path=>'/', -domain=>$domain);

  print $q->header(-charset=>$charset, -cookie=>[$cookie1, $cookie2, $cookie3]);
  &Reload($script);
}

sub LoginForm {
  print $q->header(-charset=>$charset);
  &Head($l{'login_title'});
  print <<EOT;
  </tr>
</table>

<form method=post action='$script'>
<table border=0>
  <tr>
    <td>$l{'userid'}</td>
    <td>
      <input type=text name=userid value='for_test_only\@geeksen.com' size=20>
      <input type=hidden name=mode value='list'>
    </td>
  </tr>
  <tr>
    <td>$l{'passwd'}</td>
    <td><input type=password name=passwd value='testtest' size=20></td>
  </tr>
  <tr>
    <td>$l{'server'}</td>
    <td><input type=text name=server value='jmail.geeksen.com' size=20>
    <input type=submit value='$l{'login_submit'}'></td>
  </tr>
</table>
</form>

<table border=0 width='$width'>
  <tr>
    <td><hr></td>
  </tr>
</table>

<table border=0 width='$width'>
  <tr>
    <td align=right>[
      <a href='http://www.geeksen.com/cgi/mail.txt'>Download Source</a> |
      <a href='http://www.geeksen.com'>Home</a> ]
    </td>
  </tr>
</table>
EOT
  &Tail;
}

sub Error {
  my($message) = $_[0];

  print $q->header(-charset=>$charset);
  &Head($message);
  print "</body>\n</html>";

  exit;
}

sub Reload {
  my($url) = $_[0];

  print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n\n";
  print "<html>\n<head>\n";
  print "  <title>Reload</title>\n";
  print "  <meta http-equiv='refresh' content='0;url=$url'>\n";
  print "</head>\n<body>\n</body>\n</html>";
}

sub mLOGIN {
  my($host, $sin, $tmp);

  if($userid eq '') { &Error($l{'userid_error'}); }
  if($passwd eq '') { &Error($l{'passwd_error'}); }
  if($server eq '') { &Error($l{'server_error'}); }

  socket(M, PF_INET, SOCK_STREAM, getprotobyname('tcp'));

  $host = 0;
  $host = gethostbyname($server);
  if(!$host) { &Error($l{'host_error'}); }

  $sin = sockaddr_in(110, $host);
  if(!connect(M, $sin)) { &Error($l{'connect_error'}); }

  select(M);
  $| = 1;

  select(STDOUT);
  binmode(M);

  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }

  print M "USER $userid\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }

  print M "PASS $passwd\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }
}

sub mSTAT {
  print M "STAT\r\n";
  $buf = <M>;
  $buf =~ /^\+OK\s+(\d+)\s+/i;
  $total = $1;
}

#sub mLIST {
#  if($total > 0) {
#    print M "LIST\r\n";
#    $buf = <M>;
#    if($buf =~ /^\-ERR/) { &Error($buf); }
#
#    while(1) {
#      $buf = <M>;
#      if($buf =~ /^\.\s*$/) { last; }
#      $buf =~ /(\d+) (\d+)/;
#      $s{$1} = &mSize($2);
#    }
#  }
#
#  return %s;
#}

sub mTOP {
  my($mNUM) = $_[0];
  my($TMP, $flag, $tmp, $i);
  my(@tmp);

  print M "TOP $mNUM 0\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }

  $TMP = <M>;
  while(1) {
    $buf = <M>;
    if($buf =~ /^\.\s*$/) { last; }
    $TMP .= $buf;
  }

  $m{'priority'} = '';
  $m{'attachment'} = '';
  $m{'from'} = '';
  $m{'subject'} = '';
  $m{'date'} = '';
  @tmp = split(/\n/, $TMP);

  foreach $tmp (@tmp) {
    if($tmp =~ /^from:\s*(.*)$/i || ($flag eq 'from' && $tmp =~ /^\s(.*)$/)) {
      $m{'from'} .= ' ' . &mDecodeHeader($1);
      $flag = 'from';
    }
    elsif($tmp =~ /^date:\s*(.*)$/i) {
      $m{'date'} = &mFormatDate($1);
      $flag = '';
    }
    elsif($tmp =~ /^subject:\s*(.*)$/i || ($flag eq 'subject' && $tmp =~ /^\s(.*)$/)) {
      $m{'subject'} .= &mDecodeHeader($1);
      $flag = 'subject';
    }
    elsif($tmp =~ /^X-Priority:\s*(.*)$/i) {
      $i = int($1);
      if($i == 1) { $m{'priority'} = "<font color='red'>!</font>"; }
      elsif($i == 2) { $m{'priority'} = '!'; }
      $flag = '';
    }
    elsif($tmp =~ /^Content-Type:\s*multipart\/mixed/i) {
      $m{'attachment'} = '+';
      $flag = '';
    }
    else { $flag = ''; }
  }

  if($m{'subject'} eq '') { $m{'subject'} = $l{'subject_unspecified'}; }

  foreach $tmp (keys(%m)) {
    if($tmp =~ /(from|to|cc|date|subject)/) {
      $m{$tmp} =  &mEncodeHTML($m{$tmp});
      $m{$tmp} =~ s/\r\n//g;
    }
  }

  return %m;
}

sub mRETR {
  my($mNUM, $mMIMEPART) = @_;
  my($text, $body);
  my($TMP, $tmp, $i, $j, $k);
  my($flag, $flag_mimepart);
  my($boundary, $content_type, $content_encoding, $content_id, $content_disposition, $part, $part_alternative);
  my($part_name, $part_type, $part_body, $part_size, $part_boundary, $part_encoding);
  my($part_alternative_name, $part_alternative_type, $part_alternative_body, $part_alternative_size, $part_alternative_boundary, $part_alternative_encoding);
  my(@tmp, @part, @part_alternative, @mimepart);

  print M "RETR $mNUM\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }

  $TMP = <M>;
  while(1) {
    $buf = <M>;
    if($buf =~ /^\.\s*$/) { last; }
    $TMP .= $buf;
  }

  $flag_mimepart = 0;
  @mimepart = split(/\./, $mMIMEPART);
  @mimepart = splice(@mimepart, 1, $#mimepart);

  while(1) {
    $m{'headers'} = &mEncodeHTML($TMP);
    @tmp = split(/\n/, $TMP);

    $TMP = '';
    $flag = '';
    $body = '';
    $m{'from'} = '';
    $m{'to'} = '';
    $m{'cc'} = '';
    $m{'subject'} = '';
    $m{'date'} = '';
    $m{'body'} = '';
    $m{'body_type'} = '';
    $content_type = '';

    foreach $tmp (@tmp) {
      if($tmp eq "\r") { $flag = 'body'; }

      if($flag ne 'body') {
        if($tmp =~ /^from:\s*(.*)$/i || ($flag eq 'from' && $tmp =~ /^\s(.*)$/)) {
          $m{'from'} .= ' ' . &mDecodeHeader($1);
          $flag = 'from';
        }
        elsif($tmp =~ /^to:\s*(.*)$/i || ($flag eq 'to' && $tmp =~ /^\s(.*)$/)) {
          $m{'to'} .= ' ' . &mDecodeHeader($1);
          $flag = 'to';
        }
        elsif($tmp =~ /^cc:\s*(.*)$/i || ($flag eq 'cc' && $tmp =~ /^\s(.*)$/)) {
          $m{'cc'} .= ' ' . &mDecodeHeader($1);
          $flag = 'cc';
        }
        elsif($tmp =~ /^date:\s*(.*)$/i) {
          $m{'date'} = &mFormatDate($1);
          $flag = '';
        }
        elsif($tmp =~ /^subject:\s*(.*)$/i || ($flag eq 'subject' && $tmp =~ /^\s(.*)$/)) {
          $m{'subject'} .= &mDecodeHeader($1);
          $flag = 'subject';
        }
        elsif($tmp =~ /^Content-Type:\s*(.*)$/i || ($flag eq 'content_type' && $tmp =~ /^\s(.*)$/)) {
          $content_type .= " $1";
          $flag = 'content_type';
          if($content_type =~ /^.*boundary=\s*(.*)/i) {
            $boundary = $1;
            $boundary =~ s/[\"\r\;]//g;
            $boundary =~ s/([^A-Za-z0-9 ])/\\$1/g;
            $content_type =~ s/;\s.*$//g;
            $flag = '';
          }
        }
        elsif($tmp =~ /^Content-Transfer-Encoding:\s(.*)$/i) {
          $content_encoding = $1;
        }
        else { $flag = ''; }
      }
      else { $TMP .= "$tmp\n"; }
    }

    if($content_type eq '') { $content_type = 'text/plain'; }
    if($m{'subject'} eq '') { $m{'subject'} = $l{'subject_unspecified'}; }

    $k = 0;
    $m{'attachment.count'} = 0;

    if($content_type =~ /multipart\//i) {
      $i = 0;

      @tmp = split(/\n/, $TMP);
      $TMP = '';
      $j = 0;
      $#part = -1;
      foreach $tmp (@tmp) {
        if($tmp =~ /^\-\-$boundary\-\-\r$/) { last; }
        elsif($tmp =~ /^\-\-$boundary\r$/) { $j++; }
        else { $part[$j] .= "$tmp\n"; }
      }
      @part = splice(@part, 1, $#part);

      foreach $part (@part) {
        $flag = '';
        $part_name = '';
        $part_type = '';
        $part_body = '';
        $part_size = '';
        $part_boundary = '';
        $part_encoding = '';
        $TMP = '';

        if($part =~ /^\r\n/) { $part =~ s/\r\n//; }
        @tmp = split(/\n/, $part);
        foreach $tmp (@tmp) {
          if($tmp eq "\r") { $flag = 'body'; }

          if($flag ne 'body') {
            if($tmp =~ /^Content-Type:\s*(.*)$/i || ($flag eq 'content_type' && $tmp =~ /^\s(.*)$/)) {
              $part_type .= " $1";
              $flag = 'content_type';
              if($part_type =~ /^.*boundary=\s*(.*)/i) {
                $part_boundary = $1;
                $part_boundary =~ s/[\"\r\;]//g;
                $part_boundary =~ s/([^A-Za-z0-9 ])/\\$1/g;
                $part_type =~ s/;\s.*$//g;
                $flag = '';
              }
              elsif($part_type =~ /^.*(file|name)=(.*)/i) {
                $part_name = $2;
                $part_name =~ s/[\"\r]//g;
                $part_name = &mDecodeHeader($part_name);
                $part_type =~ s/;\s.*$//g;
                $flag = '';
              }
            }
            elsif($tmp =~ /^Content-Transfer-Encoding:\s(.*)$/i) {
              $part_encoding = $1;
            }
            elsif($tmp =~ /^Content-Disposition:\s*(.*)$/i || ($flag eq 'content_disposition' && $tmp =~ /^\s(.*)$/)) {
              $content_disposition = $1;
              if($content_disposition =~ /.*filename=(.*)/i) {
                $part_name = $1;
                $part_name =~ s/[\"\r]//g;
                $part_name = &mDecodeHeader($part_name);
                $flag = '';
              }
            }
            elsif($tmp =~ /^Content-ID:\s*(.*)$/i) {
              $content_id = $1;
              if($content_id =~ /<(.*)>/) { $content_id = $1; }
              $content_id =~ s/([^A-Za-z0-9 ])/\\$1/g;
            }
          }
          else { $TMP .= "$tmp\n"; }
        }

        if($TMP =~ /^\r\n/) { $TMP =~ s/\r\n//; }
        if($part_name eq '') {
          if($part =~ /^Content-type:\s*multipart\/alternative/i) {
            @tmp = split(/\n/, $TMP);
            $TMP = '';
            $j = 0;
            $#part_alternative = -1;
            foreach $tmp (@tmp) {
              if($tmp =~ /^\-\-$part_boundary\-\-\r$/) { last; }
              elsif($tmp =~ /^\-\-$part_boundary\r$/) { $j++; }
              else { $part_alternative[$j] .= "$tmp\n"; }
            }
            @part_alternative = splice(@part_alternative, 1, $#part_alternative);

            foreach $part_alternative (@part_alternative) {
              $flag = '';
              $part_alternative_name = '';
              $part_alternative_type = '';
              $part_alternative_body = '';
              $part_alternative_size = '';
              $part_alternative_boundary = '';
              $part_alternative_encoding = '';
              $TMP = '';

              if($part_alternative =~ /^\r\n/) { $part_alternative =~ s/\r\n//; }
              @tmp = split(/\n/, $part_alternative);
              foreach $tmp (@tmp) {
                if($tmp eq "\r") { $flag = 'body'; }

                if($flag ne 'body') {
                  if($tmp =~ /^Content-Type:\s*(.*)$/i || ($flag eq 'content_type' && $tmp =~ /^\s(.*)$/)) {
                    $part_alternative_type .= " $1";
                    $part_alternative_type =~ s/;\s.*$//g;
                    $flag = 'content_type';
                  }
                  elsif($tmp =~ /^Content-Transfer-Encoding:\s(.*)$/i) {
                    $part_alternative_encoding = $1;
                  }
                }
                else { $TMP .= "$tmp\n"; }
              }

              $part_alternative_body = &mDecodeBody($part_alternative_encoding, $TMP);
              if(length($body) < length($part_alternative_body)) {
                $body = $part_alternative_body;
                $m{'body_type'} = $part_alternative_type;
              }
            }
          }
          elsif($part =~ /^Content-type:\s*message\/rfc822/i) {
            @tmp = split(/\n/, $TMP);
            $flag = '';
            foreach $tmp (@tmp) {
              if($tmp eq "\r") { last; }

              if($tmp =~ /^subject:\s*(.*)$/i || ($flag eq 'subject' && $tmp =~ /^\s(.*)$/)) {
                $part_name .= &mDecodeHeader($1);
                $flag = 'subject';
              }
              else { $flag = ''; }
            }

            if($part_name eq '') { $part_name = $l{'subject_unspecified'}; }

            $m{'attachment.' . $k} = $TMP;
            $m{'attachment.' . $k . '.name'} = $part_name;
            $m{'attachment.' . $k . '.type'} = 'message/rfc822';
            $m{'attachment.' . $k . '.size'} = length($TMP);
            $m{'attachment.' . $k . '.encoding'} = $part_encoding;
            $m{'attachment.' . $k . '.content_id'}   = '';
            $m{'attachment.count'}++;
            $k++;
          }
          else {
            $part_body = &mDecodeBody($part_encoding, $TMP);
            if(length($body) < length($part_body)) {
              $body = $part_body;
              $m{'body_type'} = $part_type;
            }
          }
        }
        else {
          $part_body = &mDecodeBody($part_encoding, $TMP);
          $part_size = length($part_body);
          if($part_name eq '') { $part_name = $l{'untitled'}; }

          $m{'attachment.' . $k} = $part_body;
          $m{'attachment.' . $k . '.name'} = $part_name;
          $m{'attachment.' . $k . '.type'} = $part_type;
          $m{'attachment.' . $k . '.size'} = $part_size;
          $m{'attachment.' . $k . '.encoding'} = $part_encoding;
          $m{'attachment.' . $k . '.content_id'}   = $content_id;
          $m{'attachment.count'}++;
          $k++;
        }
      }
    }
    else {
      $body = &mDecodeBody($content_encoding, $TMP);
      $m{'body_type'} = $content_type;
    }

    if($flag_mimepart <= $#mimepart) {
      $TMP = $m{'attachment.' . $mimepart[$flag_mimepart]};
      $flag_mimepart++;
    }
    else { last; }
  }

  $m{'bgcolor'} = '#ffffff';
  if($m{'body_type'} =~ /text\/html/i) {
    $body =~ s/=(\"|\'|)mailto:([A-Za-z0-9-_.]+\@[A-Za-z0-9-.]+)(\"|\'|)/=$1$script\?mode=form&to=$2$3/gi;
    $body =~ s/=(\"|\'|)$script\?mode=form&to=(.*)\?subject=(.*)(\"|\'|)/=$1$script\?mode=form&to=$2&subject=$3$4/gi;

    $m{'background'} = '';
    for($i = 0; $i < $m{'attachment.count'}; $i++) {
      $content_id = $m{'attachment.' . $i . '.content_id'};

      if($content_id ne '') {
        $part_name  = "$script?mode=download&amp;num=$mNUM&amp;mimepart=$mMIMEPART&amp;time=$time";
        $part_name .= "&amp;attachment=$i/" . $m{'attachment.' . $i . '.name'};
        $part_name =~ s/ /%20/g;

        if($body =~ /<body(.|\n)*background=(\"|'|)cid\:$content_id/i) {
          $m{'background'} = $part_name;
          $body =~ s/background=(\"|'|)cid\:$content_id//g;
        }
        $body =~ s/cid\:$content_id/$part_name/g;
      }
    }

    if($body =~ /<body.*bgcolor=(\"|'|)([#A-Za-z0-9]+)(\"|'|\s|>|)/i) {
      $m{'bgcolor'} = $2;
    }

    $body =~ s/<\/?(base|html|head|title|meta|link|body)[^>]*\>//gi;
  }
  else {
    $body =  &mEncodeHTML($body);
    $body =~ s/\n/<br>\n/g;
  }

  $body =~ s/(<|&lt;)([A-Za-z0-9-_.]+\@[A-Za-z0-9-.]+)(>|&gt;)/&lt;<a href='$script\?mode=form&to=$2'>$2<\/a>&gt;/gi;
  $body =~ s/([^=|\"|'|>|&lt;])([\s|](http|https|ftp):\/\/[A-Za-z0-9-.]+[A-Za-z0-9:&#@=_~%;\?\/\.\+-]+)/$1<a href='$2' target='_blank'>$2<\/a>/gi;

  $body =~ s/<script/<noscript/gi;
  $body =~ s/<\/script>/<\/noscript>/gi;
  $body =~ s/<style/<\!--style/gi;
  $body =~ s/<\/style>/<\/style-->/gi;
  $body =~ s/document\.cookie/document_cookie/g;

  $m{'body'} = $body;

  foreach $tmp (keys(%m)) {
    if($tmp =~ /(from|to|cc|date|subject)/) {
      $m{$tmp} =  &mEncodeHTML($m{$tmp});
      $m{$tmp} =~ s/\r\n//g;
    }
  }

  return %m;
}

sub mDELE {
  my($mNUM) = $_[0];

  print M "DELE $mNUM\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }
}

sub mQUIT {
  print M "QUIT\r\n";
  $buf = <M>;
  if($buf =~ /^\-ERR/) { &Error($buf); }

  close(M);
}

sub mDecodeHeader {
  my($str1) = $_[0];
  my($str2);

  if($str1 =~ /=\?.*\?B\?(.*)\?=/i) {
    $str2 =  &mDecodeBase64($1);
    $str1  =~ s/=\?.*\?B\?.*\?=/$str2/i;
  }
  elsif($str1 =~ /=\?.*\?Q\?(.*)\?=/i) {
    $str2 =  &mDecodeQuotedPrintable($1);
    $str1  =~ s/=\?.*\?Q\?.*\?=/$str2/i;
  }
  $str1 =~ s/\r\n//g;

  return $str1;
}

sub mDecodeBody {
  my($encoding, $str) = @_;

  if($encoding =~ /base64/i) {
    return &mDecodeBase64($str);
  }
  elsif($encoding =~ /quoted-printable/i) {
    return &mDecodeQuotedPrintable($str);
  }
  return $str;
}

sub mEncodeHTML {
  my($str) = $_[0];
  $str =~ s/</&lt;/g;
  $str =~ s/>/&gt;/g;
  return $str;
}

sub mDecodeHTML {
  my($str) = $_[0];
  $str =~ s/&lt;/</gi;
  $str =~ s/&gt;/>/gi;
  return $str;
}

sub mEncodeBase64 {
  my($str1) = $_[0];
  my($str2, $padding);

  while ($str1 =~ /(.{1,45})/gs) {
    $str2 .= substr(pack('u', $1), 1);
    chop($str2);
  }
  $str2 =~ tr|` -_|AA-Za-z0-9+/|;

  $padding = (3 - length($str1) % 3) % 3;
  if($padding) { $str2 =~ s/.{$padding}$/'=' x $padding/e; }
  $str2 =~ s/(.{1,76})/$1\r\n/g;

  return $str2;
}

sub mDecodeBase64 {
  my($str1) = $_[0];
  my($str2, $tmp, $i);

  for($i = 0; $i < (length($str1) % 4); $i++) { chop($str1); }
  $str1 =~ tr|A-Za-z0-9+=/||cd;
  $str1 =~ s/=+$//;
  $str1 =~ tr|A-Za-z0-9+/| -_|;
  while($str1 =~ /(.{1,60})/gs) {
    $tmp = chr(32 + length($1) * 3 / 4);
    $str2 .= unpack("u", $tmp . $1 );
  }

  return $str2;
}

sub mDecodeQuotedPrintable {
  my($str1) = $_[0];
  my($str2);

  $str1 =~ s/\s+(\r?\n)/$1/g;
  $str1 =~ s/=\r?\n//g;
  $str2 = $str1;
  $str2 =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge;

  return $str2;
}

sub mGetDate {
  my(%months) = ('01', 'Jan', '02', 'Feb', '03', 'Mar', '04', 'Apr', '05', 'May', '06', 'Jun', '07', 'Jul', '08', 'Aug', '09', 'Sep', '10', 'Oct', '11', 'Nov', '12', 'Dec');
  my(@weeks) = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
  my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);

  if($timezone eq 'GMT') {
    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time);
  }
  else {
    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
  }

  $year += 1900;
  $mon  += 1;
  if($mon  < 10) { $mon  = "0$mon";  }
  if($mday < 10) { $mday = "0$mday"; }
  if($hour < 10) { $hour = "0$hour"; }
  if($min  < 10) { $min  = "0$min";  }
  if($sec  < 10) { $sec  = "0$sec";  }

  return "$weeks[$wday], $mday $months{$mon} $year $hour:$min:$sec $timezone";
}

sub mFormatDate {
  my($time) = $_[0];
  my($day, $month, $year, $hour_min, $gmt_interval);
  my(%months) = ('Jan', '01', 'Feb', '02', 'Mar', '03', 'Apr', '04', 'May', '05', 'Jun', '06', 'Jul', '07', 'Aug', '08', 'Sep', '09', 'Oct', '10', 'Nov', '11', 'Dec', '12');

  if($time =~ /\,\s+/) { $time =~ s/.*\,\s+//; }
  ($day, $month, $year, $hour_min, $gmt_interval) = split(/ /, $time);
  if($day < 10) { $day = '0' . int($day); }

  return "$year-$months{$month}-$day $hour_min $gmt_interval";
}

sub mSize {
  my($size) = $_[0];

  if($size >= 1048576) { return int($size / 1048576) . 'M' }
  elsif($size >= 1024) { return int($size / 1024) . 'K'    }
  else { return '1K' }
}

sub mLocale {
  if($locale eq 'ko') {
    $l{'login_title'} = 'α';

    $l{'userid'} = '̵';
    $l{'passwd'} = 'йȣ';
    $l{'server'} = '޴¸ϼ';

    $l{'logout'} = 'α׾ƿ';
    $l{'headers'} = '';
    $l{'delete'} = '';
    $l{'compose'} = '';
    $l{'forward'} = '';
    $l{'reply_all'} = 'üȸ';
    $l{'reply'} = 'ȸ';
    $l{'reload'} = 'ΰħ';

    $l{'total'} = '޽';
    $l{'page'} = '';
    $l{'no'} = 'ȣ';
    $l{'from'} = ' ';
    $l{'sender'} = ' ';
    $l{'to'} = '޴ ';
    $l{'cc'} = '';
    $l{'bcc'} = '';
    $l{'date'} = '¥';
    $l{'subject'} = '';
    $l{'smtp'} = ' ';
    $l{'attachment'} = '÷';
    $l{'original_message'} = ' ޽';
    $l{'original_message_included'} = ' ޽ ÷εǾϴ.';

    $l{'prev'} = '';
    $l{'next'} = '';
    $l{'list'} = 'Ϻ';

    $l{'login_submit'} = 'α';
    $l{'wanna_delete'} = 'Ͻðڽϱ?';
    $l{'delete_submit'} = '  ';
    $l{'send_submit'} = '      ';
    $l{'cancel_button'} = '       ';

    $l{'userid_error'} = '̵ ʿմϴ.';
    $l{'passwd_error'} = 'йȣ ʿմϴ.';
    $l{'server_error'} = '  ʿմϴ.';
    $l{'from_error'} = '  ̸ ʿմϴ.';
    $l{'to_error'} = '޴  ʿմϴ.';
    $l{'smtp_error'} = '   ʿմϴ.';
    $l{'sendmail_error'} = ' α׷   ϴ.';
    $l{'body_error'} = ' ʿմϴ.';
    $l{'host_error'} = ' IP ּҸ ã  ϴ.';
    $l{'connect_error'} = '   ϴ.';

    $l{'untitled'} = '̸';
    $l{'subject_unspecified'} = '';
  }
  elsif($locale eq 'en') {
    $l{'login_title'} = 'Login';

    $l{'userid'} = 'UserID';
    $l{'passwd'} = 'Passwd';
    $l{'server'} = 'POP Server';

    $l{'logout'} = 'Logout';
    $l{'headers'} = 'Headers';
    $l{'delete'} = 'Delete';
    $l{'compose'} = 'Compose';
    $l{'forward'} = 'Forward';
    $l{'reply_all'} = 'Replay All';
    $l{'reply'} = 'Reply';
    $l{'reload'} = 'Reload';

    $l{'total'} = 'Total';
    $l{'page'} = 'Page';
    $l{'no'} = 'No.';
    $l{'from'} = 'From';
    $l{'sender'} = 'From';
    $l{'to'} = 'To';
    $l{'cc'} = 'CC';
    $l{'bcc'} = 'BCC';
    $l{'date'} = 'Date';
    $l{'subject'} = 'Subject';
    $l{'smtp'} = 'SMTP Server';
    $l{'attachment'} = 'Attachment';
    $l{'original_message'} = 'Original Message';
    $l{'original_message_included'} = 'Original Message Included.';

    $l{'prev'} = 'Prev';
    $l{'next'} = 'Next';
    $l{'list'} = 'List';

    $l{'login_submit'} = 'Login';
    $l{'wanna_delete'} = 'Wanna Delete?';
    $l{'delete_submit'} = 'Delete Marked';
    $l{'send_submit'} = '    Send    ';
    $l{'cancel_button'} = '  Cancel  ';

    $l{'userid_error'} = 'UserID Must Be Specified.';
    $l{'passwd_error'} = 'Passwd Must Be Specified.';
    $l{'server_error'} = 'Server Must Be Specified.';
    $l{'from_error'} = 'From Must Be Specified.';
    $l{'to_error'} = 'To Must Be Specified.';
    $l{'smtp_error'} = 'SMTP Server Must Be Specified.';
    $l{'sendmail_error'} = 'Could NOT Open SendMail.';
    $l{'body_error'} = 'Body Must Be Specified.';
    $l{'host_error'} = 'Could NOT Find An IP Address For The Host.';
    $l{'connect_error'} = 'Could NOT Connect To Server.';

    $l{'untitled'} = 'Untitled';
    $l{'subject_unspecified'} = 'Unspecified';
  }

  return %l;
}
